import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import '../sys/Command.dart';

class OutputStream {
  final Socket _socket;

  OutputStream(this._socket);

  void writeRawBytes(List<int> bytes) {
    _socket.add(bytes);
  }

  void writeBytes(List<int> bytes) {
    writeInt(bytes.length);
    _socket.add(bytes);
  }

  void writeCommand(Command cmd) {
    writeString(cmd.toString());
  }

  void writeString(String str) {
    final lastCharIndex = str.length - 1;

    if (str.codeUnitAt(lastCharIndex) == 10)
      str = str.substring(0, lastCharIndex);
    //print("ToSend: ${str}");
    //print("IntCodeUnits: ${Uint8List.fromList(str.codeUnits).buffer.asInt8List()}");
    writeBytes(str.codeUnits);
  }

  void writeBoolean(bool b) {
    _socket.add(List.of([b ? 1 : 0]));
  }

  void writeInt(int value) {
    var intList = Uint8List(4);
    intList[0] = (value >> 24 & 0xFF);
    intList[1] = (value >> 16 & 0xFF);
    intList[2] = (value >> 8 & 0xFF);
    intList[3] = (value >> 0 & 0xFF);
    _socket.add(intList);
  }
}
