import 'dart:math';
import 'CmdOrders.dart';

class Command {
  String order;
  List<String> options;

  static final Command UNKNOWN_CMD = new Command.fromString(CmdOrders.UNKNOWN);

  Command(String order, List<dynamic> options) {
    this.order = order;
    this.options = <String>[];
    addOptions(options);
  }

  // ojo con letra+"
  Command.fromString(String strCmd) {
    strCmd = strCmd.trim();

    if (strCmd.endsWith(String.fromCharCode(10)))
      strCmd = strCmd.substring(0, strCmd.length-1);

    bool containsSpace = strCmd.contains(" ");
    order = containsSpace ? strCmd.split(" ")[0] : strCmd.trim();
    options = <String>[];

    if (containsSpace)
      buildOptionsFromString(strCmd);
  }

  // puede tener problemas con el tema de los espacios
  void buildOptionsFromString(String strCmd) {
    final List<String> splitCmd = strCmd.split(" ");

    if (strCmd.contains("\"")) {
      final List<int> sbData = <int>[];
      List<int> sbElement = <int>[];

      for (int i = 1; i < splitCmd.length; i++) {
        sbData.addAll(splitCmd[i]
            .trim()
            .codeUnits);
        sbData.add(' '.codeUnitAt(0));
      }

       String firstString = String.fromCharCodes(sbData).trim();
        List<int> firstStrChars = firstString.codeUnits;
        sbData.clear();

        List<String> listElements = <String>[];
        bool foundQuotation = firstStrChars[0] == '\"';

        int elementChar;
        for (int i = 0; i < firstStrChars.length; i++) {
          elementChar = firstStrChars[i];
          sbElement.add(elementChar);
          if (foundQuotation) {
            if (elementChar == '\"' &&
                (i < firstStrChars.length - 1 && firstStrChars[i + 1] == ' ')) {
              foundQuotation = false;
              if (sbElement.length > 0) {
                listElements.add(String.fromCharCodes(sbElement));
                sbElement.clear();
              }
            }
          }
          else {
            if (elementChar == ' ' && (i < firstStrChars.length - 1 &&
                firstStrChars[i + 1] == '\"')) {
              foundQuotation = true;
              if (sbElement.length > 0) {
                listElements.add(String.fromCharCodes(sbElement));
                sbElement.clear();
              }
            }
          }
        }

        if (sbElement.length > 0)
          listElements.add(String.fromCharCodes(sbElement));

        sbElement = null;

        listElements.forEach((element) {
          if (element.startsWith("\""))
            options.add(element);
          else {
            if (element.trim().contains(" ")) {
              final List<String> elementSplit = element.trim().split(" ");
              options.addAll(elementSplit);
            }
            else if (element
                .trim()
                .length > 0)
              options.add(element.trim());
          }
        });
      }
    else
      for (int i = 1; i < splitCmd.length; i++)
        options.add(splitCmd[i]);
  }

  bool hasOptions() {
    return !options.isEmpty;
  }

  int getOptionsCount() {
    return hasOptions() ? options.length : 0;
  }

  void addOptions(List<dynamic> options) {
    if (options != null && options.length > 0) {
      for (int i = 0; i < options.length; i++) {
        if (options[i] is String)
          this.options.add("\"" + options[i].toString() + "\"");
        else
          this.options.add(options[i].toString());
      }
    }
  }

  String getOptionAt(int index) {
    if (hasOptions() && getOptionsCount() > index) {
      String option = options[index].trim();
      return option.startsWith("\"") && option.endsWith("\"")
          ? option.substring(1, option.length - 1)
          : option;
    }
    else
      return null;
  }

  String getTrimmedOptionAt(int index) {
    String optionAt = getOptionAt(index);
    return optionAt != null ? optionAt.trim() : null;
  }

  num getOptionAsNumber(int index) {
    String option = getOptionAt(index);
    try {
      return option == null ? null : double.parse(option);
    } catch (e) {
      return null;
    }
  }

  String getOrder() {
    return order;
  }

  List<String> getOptionsCopy() {
    return List.of(options);
  }

  String getOptionsAsString() {
    return getOptionsAsStringRange(0, options.length);
  }

  String getOptionsAsStringRange(int from, int to) {
    if (hasOptions()) {
      final List<int> sbOptions = <int>[];
      for (int i = max(0, from);
      i < min(to, options.length); i++) {
        sbOptions.addAll(getOptionAt(i).codeUnits);
        sbOptions.add(' '.codeUnitAt(0));
      }
      sbOptions.removeAt(sbOptions.length - 1);
      String.fromCharCodes(sbOptions);
    }
    return null;
  }

  String getOptionsAsStringFrom(int from) {
    return getOptionsAsStringRange(from, getOptionsCount());
  }

  String toString() {
    List<String> sbCmd = <String>[];
    sbCmd.add(order);
    if (hasOptions())
      for (int i = 0; i < options.length; i++) {
        sbCmd.add(' ');
        sbCmd.add(options[i]);
      }

    StringBuffer sbToString = new StringBuffer();
    sbCmd.forEach((element) {
      sbToString.write(element);
    });
    return sbCmd.join();
  }

}